//
//  FilBehandler.m
//  HeisApp
//
//  Created by Sturla Sandvei on 4/21/11.
//  Copyright 2011 NTNU. All rights reserved.
//

#import "FilBehandler.h"
#import "HeisAppDelegate.h"

@implementation FilBehandler

// Tar bort kvitteringsmelding på skjermen etter at timeren har gått ut
- (void)stopp:(NSTimer *)mldTimer {
	[[mldTimer userInfo] dismissWithClickedButtonIndex:0 animated:YES];
	[mldTimer invalidate];
    
}

// Her bygges linje for linje opp, dvs denne kalles for hver linje i tabellen
-(UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    static NSString *MyIdentifier = @"Cell";
    
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:MyIdentifier];
    
    if (cell == nil) {
        cell = [[[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:MyIdentifier] autorelease];
    }
    
    NSString *celleVerdi;
    
    // Sett inn navn fra tabellen til linjene i tableView
    if (indexPath.row < [tabellInnhold count])
    {
        celleVerdi = [tabellInnhold objectAtIndex:indexPath.row];
        cell.textLabel.text = celleVerdi;
    }
    else
    {
        celleVerdi = @"<tom>";
        cell.textLabel.text = celleVerdi;
    }
    
    return cell;
}

// Vi har bare en seksjon i tabellen vår, dvs ikke splittet i f.eks kategorier
- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
    return 1;
}

// Antall rader i tabellen er likt antall filer i csv-tabellen vår
- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{    
    return [tabellInnhold count];
}


// Oppretter headerfelt for tabellen
- (UIView *)tableView:(UITableView *)tableView viewForHeaderInSection:(NSInteger)section
{
    if(section != 0) 
    {
        UIView *view = [[[UIView alloc] initWithFrame:CGRectMake(10, 10, 100, 30)] autorelease];
        view.backgroundColor = [UIColor redColor];
        
        return view;
    } 
    else
        return tableView.tableHeaderView;

}

// Legger inn tekst som skal ligge i tabellheaderfeltet
- (NSString *)tableView:(UITableView *)tableView titleForHeaderInSection:(NSInteger)section
{
    return @"Forsøk lagret på telefonen";
}

// Brukeren har pekt på en fil og da skal den settes som aktiv fil i systemet, dvs grafen skal vise
// innholdet fra denne filen og sending av e-post skal ha denne filen som vedlegg.
- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    int rad = indexPath.row;
    
    if (rad != NSNotFound)
    {
        HeisAppDelegate *hovedDelegat = (HeisAppDelegate *)[[UIApplication sharedApplication] delegate];
        
        // Finner katalogen hvor filene er lagret
        NSArray *kataloger = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDirectory, YES);
        NSString *katalog = [kataloger objectAtIndex:0];
        
        // Setter global variabel csvFil med komplett filnavn for aktiv fil dvs inkl kataloger
        // NB! Husk at lokal tabell er snudd på hodet ift global filtabell! Må derfor "fikse" indeksen.
        hovedDelegat.csvFil = [katalog stringByAppendingPathComponent:[hovedDelegat.filTabell objectAtIndex:[tabellInnhold count]-rad-1]];
        
        // Legger ut en kvitteringsmelding
        UIAlertView *melding = [[UIAlertView alloc]
                                initWithTitle:@"Aktiv måling er nå" message:[tabellInnhold objectAtIndex:rad] 
                                delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil];
        
        // Starter en timer for å fjerne meldingen på skjermen hvis brukeren ikke trykker OK
        [NSTimer scheduledTimerWithTimeInterval:1 target:self selector:@selector(stopp:) userInfo:melding repeats:NO];
        
        [melding show];
        [melding release];
        
    }
}

// Kalles en gang for hver celle med innhold når vi er over i editingmodus
- (UITableViewCellEditingStyle)tableView:(UITableView *)tableView editingStyleForRowAtIndexPath:(NSIndexPath *)indexPath
{
    if (self.editing)
        return UITableViewCellEditingStyleDelete;
    return UITableViewCellEditingStyleNone;
}

// Sletting av filer i tabellen, kalles når bruker trykker på DELETE og CONFIRM
- (void)tableView:(UITableView *)tableView commitEditingStyle:(UITableViewCellEditingStyle)editingStyle forRowAtIndexPath:(NSIndexPath *)indexPath
{
    if (editingStyle == UITableViewCellEditingStyleDelete)
    {
        // Finner indeks i tabellview som brukeren vil slette
        int rad = indexPath.row;
        
        if (rad != NSNotFound)
        {
            HeisAppDelegate *hovedDelegat = (HeisAppDelegate *)[[UIApplication sharedApplication] delegate];

            // Sletter filen i telefonens minne.
            // Må fikse på indeksen her da lokal tabell er snudd på hodet ift global filtabell
            NSFileManager *fileManager = [NSFileManager defaultManager];
            NSArray *kataloger = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDirectory, YES);
            NSString *katalog = [kataloger objectAtIndex:0];
            NSString *fil = [katalog stringByAppendingPathComponent:[hovedDelegat.filTabell objectAtIndex:[tabellInnhold count]-rad-1]];
            [fileManager removeItemAtPath:fil error:Nil];

            [tableView beginUpdates];
            
            // Sletter fra tabellview dvs de linjene som vises på skjermen
            [tableView deleteRowsAtIndexPaths:[NSArray arrayWithObject:indexPath]  withRowAnimation:UITableViewRowAnimationFade];
            
            // Sletter fra global filtabell
            // Må fikse på indeksen her da lokal tabell er snudd på hodet ift global filtabell
            [hovedDelegat.filTabell removeObjectAtIndex:[tabellInnhold count]-rad-1];
            
            // Slett fra lokal tabell (her stemmer indeksen)
            [tabellInnhold removeObjectAtIndex:rad];
            
            [tableView endUpdates];
            
            // Nullstiller global variabel csvFil hvis det var aktiv fil vi slettet
            if ( [fil isEqualToString:hovedDelegat.csvFil] )
                hovedDelegat.csvFil = @"ingen";
        }
    }
}

// Alle linjer i datatabellen er editerbar for oss (dvs alle filer kan slettes)
- (BOOL)tableView:(UITableView *)tableView canEditRowAtIndexPath:(NSIndexPath *)indexPath
{
    if (indexPath.row < [tabellInnhold count])
        return TRUE;
    return FALSE;
}


// Endre til norsk tekst på knappen for å bekrefte sletting
- (NSString *)tableView:(UITableView *)tableView titleForDeleteConfirmationButtonForRowAtIndexPath:(NSIndexPath *)indexPath
{
    return @"Bekreft";
}


- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil
{
    self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
    if (self) {
        // Custom initialization
    }
    return self;
}

// Går inn i editeringsmodus på skjermen
- (void)setEditing:(BOOL)editing animated:(BOOL)animated
{
    [super setEditing:editing animated:animated];
    [self.view setEditing:editing animated:animated];

    self.editButtonItem.enabled = YES;

    if (editing)
    {
        // Endre tekst på DONE-knapp til norsk
        self.editButtonItem.title = @"Ferdig";        
    }
    else
    {
        self.editButtonItem.enabled = YES;
        // Endre tekst på EDIT-knapp til norsk
        self.editButtonItem.title = @"Slett filer";
    }

}

- (void)dealloc
{
    [tabellInnhold release];
    [super dealloc];
}

- (void)didReceiveMemoryWarning
{
    // Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
    
    // Release any cached data, images, etc that aren't in use.
}

- (void)viewDidLoad
{
    [super viewDidLoad];
}

- (void)viewDidAppear:(BOOL)animated
{
    [self.view reloadInputViews];
}

// Oppstart av skjermbildet (kalles hver gang - ikke bare første)
- (void)viewWillAppear:(BOOL)animated
{    
    self.title = @"Filer";
    // Legg inn knapp i toppen for å kunne gå i edit-modus
    // og slette lagrede filer. Norsk tekst på knappen.
    self.navigationItem.leftBarButtonItem = self.editButtonItem;
    self.navigationItem.leftBarButtonItem.title = @"Slett filer";
    
    // Opprett en egen tabell som holder styr på innholder på skjermen
    tabellInnhold = [[NSMutableArray alloc] init];
    
    HeisAppDelegate *hovedDelegat = (HeisAppDelegate *)[[UIApplication sharedApplication] delegate];
    
    NSString *filnavn;
    
    // Speiler den global fil-tabellen slik at siste fil kommer øverst i tabellen
    // som skal vises på skjermen (siste først).
    for (int i=[hovedDelegat.filTabell count]; i > 0 ; i--)
    {
        // Fyll inn lokal tabell med alle filer som er lagret på app'en
        // Legg inn en tekst som er litt "finere" enn filnavnet
        filnavn = [NSString stringWithFormat:@" %@, kl. %@",
                   [[hovedDelegat.filTabell objectAtIndex:i-1] substringToIndex:10],
                   [[hovedDelegat.filTabell objectAtIndex:i-1] substringWithRange:NSMakeRange(11, 8)]];
        [tabellInnhold addObject:filnavn];
    }    
    
    // Frisk opp innholdet på skjermen
    [self.view reloadData];
    
}


- (void)reloadInputViews
{
    [super reloadInputViews];
}

- (void)viewDidUnload
{
    [super viewDidUnload];
    // Release any retained subviews of the main view.
    // e.g. self.myOutlet = nil;
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
    // Return YES for supported orientations
    return (interfaceOrientation == UIInterfaceOrientationPortrait);
}

@end
