//
//  Graf.m
//  HeisApp
//
//  Created by Sturla Sandvei on 4/19/11.
//  Copyright 2011 NTNU. All rights reserved.
//

// Gråtone-frager
CGColorRef CreateDeviceGrayColor(CGFloat w, CGFloat a)
{
	CGColorSpaceRef gray = CGColorSpaceCreateDeviceGray();
	CGFloat comps[] = {w, a};
	CGColorRef color = CGColorCreate(gray, comps);
	CGColorSpaceRelease(gray);
	return color;
}

// RGB-farger
CGColorRef CreateDeviceRGBColor(CGFloat r, CGFloat g, CGFloat b, CGFloat a)
{
	CGColorSpaceRef rgb = CGColorSpaceCreateDeviceRGB();
	CGFloat comps[] = {r, g, b, a};
	CGColorRef color = CGColorCreate(rgb, comps);
	CGColorSpaceRelease(rgb);
	return color;
}

// Gråfargen som er bakgrunn på grafen
CGColorRef graphBackgroundColor()
{
	static CGColorRef c = NULL;
	if(c == NULL)
	{
		c = CreateDeviceGrayColor(0.6, 1.0);
	}
	return c;
}

// Gråsvart som er fargen på hjelpelinjene i grafen
CGColorRef graphLineColor()
{
	static CGColorRef c = NULL;
	if(c == NULL)
	{
		c = CreateDeviceGrayColor(0.5, 1.0);
	}
	return c;
}

// Rødfargen som er fargen på selve grafen
CGColorRef graphYColor()
{
	static CGColorRef c = NULL;
	if(c == NULL)
	{
		c = CreateDeviceRGBColor(1.0, 0.0, 0.0, 1.0);
	}
	return c;
}

// Sort som er fargen på aksene i grafen
CGColorRef akseColor()
{
	static CGColorRef c = NULL;
	if(c == NULL)
	{
		c = CreateDeviceRGBColor(0.0, 0.0, 0.0, 1.0);
	}
	return c;
}

#import "Graf.h"

@implementation Graf

- (id)initWithFrame:(CGRect)frame
{
    if ((self = [super initWithFrame:frame]))
        self.backgroundColor = [UIColor colorWithCGColor:graphBackgroundColor()];
    return self;
}

- (void)drawRect:(CGRect)rect {

    // En del interne hjelpevariabler
    float maks = 0.0, min = 0.0;
    float yValues[50000];
    float xMultiplier, yMultiplier, maxVerdi;
    int pointCount=0;
    NSString *dato, *startTid;
    
    HeisAppDelegate *hovedDelegat = (HeisAppDelegate *)[[UIApplication sharedApplication] delegate];
    
    // Henter data fra aktuell CSV-fil og leser inn i en lokal tabell
    // Sjekker først om det er satt en aktiv fil
    if ( ! [hovedDelegat.csvFil isEqualToString:@"ingen"] ) {
        // Leser hele filen inn i en string
        NSString* filInnhold = [NSString stringWithContentsOfFile:hovedDelegat.csvFil encoding:NSUTF8StringEncoding error:Nil];
        
        // Splitter opp filen på komma (CSV-fil) og putter feltene inn i en array
        NSArray* splittInnhold = [filInnhold componentsSeparatedByString:@","];
        
        dato = [splittInnhold objectAtIndex:3];
        startTid = [splittInnhold objectAtIndex:5];
        
        // Her er det viktig å vite nøyaktig hvordan CSV-filen er oppdelt i felter (heading og data-linjer).
        // Første måleverdi kommer i felt nr: (ant felt i heading)+(ant felt per linje før måleverdi)-1
        // Modulus det skal regnes med er antall felt per linje med data, og om denne skal bli 0, 1, 2 osv
        //       må beregnes i hver tilfelle ;-)
        
        // Gå gjennom tabellen som inneholder alle feltene fra CSV-filen
        // Start med første dataverdi
        for (int i=13; i < [splittInnhold count]; i++)
        {
            // Vi er bare interessert i hver x-te felt dvs felt med måledata
            if (i % 3 == 1)
            {
                // Lagre dataverdiene i en lokal tabell
                yValues[pointCount] = [[splittInnhold objectAtIndex:i] floatValue];
                // Hold styr på max og min verdier
                if ( yValues[pointCount] > maks ) { maks = yValues[pointCount]; }
                else { if ( yValues[pointCount] < min ) { min = yValues[pointCount]; } }
                // Tell opp antall målinger
                pointCount++;
            }
        }
                
        // Finn ekstremverdi for Y i datasett slik at grafen kan tilpasses i høyde
        // (vi ønsker at skjermen skal fylles i størst mulig grad)
        maxVerdi = MAX(maks, -min);
        
        // Finn skalar slik at y-verdiene fyller skjermen (300 / 2 = 150)
        yMultiplier =  (float) 150 / maxVerdi;
        
        // Finn skalar slik at x-verdiene akkurat fyller bredde på skjermen (395-10 = 385)
        if ( pointCount != 0 ) { xMultiplier = (float) 385 / pointCount; }

    
    
    }
        
	// Opprett kontekst som vi kan tegne og skrive til :-)
    // NB! Skal alltid skrive noe til skjerm så denne er allmenn gyldig uansett filstatus
	CGContextRef contxt = UIGraphicsGetCurrentContext();    
    CGContextTranslateCTM(contxt, 0.0, 400.0);
    // Roter slik at grafen blir vist i landskapsmodus på skjermen
    CGContextRotateCTM(contxt, -1.570796);  // PI_halve aka 90 grader

    
    // Hvis det er noe data å skrive ut skal en del nips legge på i bildet
    if ( pointCount != 0 )
    {
        // Tegn hjelpelinjer paralelt med x-aksen
        // Max ligger i området 0.5 og 5.0 (rundt 1 på "normale" heiser)
        // Vi ønsker å tegne 4 hjelpelinjer
        int step, faktor;
        NSString* filler;
        
        if (maxVerdi >= 2) {
            step = (int) maxVerdi / 2;
            faktor = step*yMultiplier;
            filler = @"";
        }
        else if (maxVerdi >= 0.2) {
            step = (int) (10*maxVerdi) / 2;
            faktor = (step*yMultiplier)/10;
            filler = @".";
        }
        else if (maxVerdi >= 0.02) {
            step = (int) (100*maxVerdi) / 2;
            faktor = (step*yMultiplier)/100;
            filler = @".0";
        }
        else {
            step = (int) (1000*maxVerdi) / 2;
            faktor = (step*yMultiplier)/1000;
            filler = @".00";
        }
        
        CGContextSetStrokeColorWithColor(contxt, graphLineColor());
        
        for (int i = 1  ;  i < 3 ; i++) 
        {
            
            CGContextMoveToPoint(contxt, 10, 160-i*faktor);
            CGContextAddLineToPoint(contxt, 395, 160-i*faktor);
            CGContextStrokePath(contxt);
            
            CGContextMoveToPoint(contxt, 10, 160+i*faktor);
            CGContextAddLineToPoint(contxt, 395, 160+i*faktor);
            CGContextStrokePath(contxt);
            
        }
        
        
        // Tegn nummer på andreaksen
        [[NSString stringWithFormat:@"%@%i",filler,step] drawInRect:CGRectMake( -22, 154-faktor, 30, 15) 
                                                           withFont:[UIFont systemFontOfSize:10] 
                                                      lineBreakMode:UILineBreakModeWordWrap 
                                                          alignment:UITextAlignmentRight];
        
        [[NSString stringWithFormat:@"-%@%i",filler,step] drawInRect:CGRectMake( -22, 154+faktor, 30, 15) 
                                                            withFont:[UIFont systemFontOfSize:10] 
                                                       lineBreakMode:UILineBreakModeWordWrap 
                                                           alignment:UITextAlignmentRight];
        
        // Tegn nummer på førsteaksen - et "rundt" antall sekunder til høyre på aksen
        // pointCount er maxverdien dvs den kan skrives i ytterste høyre posisjon
        // Sjekker om noen har klart å kjøre en måling på under 1 sek, i så fall settes 1 inn likevel :-)
        if ( pointCount/20 == 0 )
            [@"1" drawInRect:CGRectMake(380, 170, 45, 30) withFont:[UIFont systemFontOfSize:11]
                                                        lineBreakMode:UILineBreakModeTailTruncation 
                                                            alignment:UITextAlignmentLeft];
        else
            [[NSString stringWithFormat:@"%i", pointCount/20] drawInRect:CGRectMake(380, 170, 45, 30) 
                                                       withFont:[UIFont systemFontOfSize:11] 
                                                  lineBreakMode:UILineBreakModeTailTruncation 
                                                      alignment:UITextAlignmentLeft];
        
        // Kanskje kjekt med litt mer enn ett nummer også...? ## Lagt inn etter demo#1.
        // Sjekk om målingen er under 2 sekunder. I såfall settes ikke noe tall inn her.
        if ( pointCount/40 != 0 )
            [[NSString stringWithFormat:@"%i", pointCount/40] drawInRect:CGRectMake(195, 170, 45, 30) 
                                                                  withFont:[UIFont systemFontOfSize:11] 
                                                             lineBreakMode:UILineBreakModeTailTruncation 
                                                                 alignment:UITextAlignmentLeft];
        
        // Skriv inn tittel på grafen med dato og klokkeslett
        [[NSString stringWithFormat:@"Heisforsøk %@ kl. %@", dato, startTid] drawInRect:CGRectMake(100, 5, 200, 20) 
                                                                               withFont:[UIFont systemFontOfSize:11] 
                                                                          lineBreakMode:UILineBreakModeTailTruncation 
                                                                              alignment:UITextAlignmentLeft];
        
        [filler release];
        
    }
    else
        // Legg ut en liten epistel hvis det mangler data
    {
        [@"Har ingen data å lage graf ut av...\nDu må enten utføre et forsøk eller velge en fil først." 
                                                                            drawInRect:CGRectMake(40, 80, 300, 200) 
                                                                               withFont:[UIFont systemFontOfSize:16] 
                                                                          lineBreakMode:UILineBreakModeTailTruncation 
                                                                              alignment:UITextAlignmentLeft];
    }

   
    // Hovedaksene med måleenheter tenger vi uansett så det ser litt OK ut selv uten data
    
    CGContextSetStrokeColorWithColor(contxt, akseColor()); // Sett farge som grafen skal ha

    // Tegn førsteakse med måleenhet
    CGContextMoveToPoint(contxt, 10, 160); // Start ny strek fra dette punkt
    CGContextAddLineToPoint(contxt, 395, 160); // Lag en linje fra forrige punkt og hit
    CGContextStrokePath(contxt); // Og så tegner vi langs linjen
    [[NSString stringWithFormat:@"[sek]"] drawInRect:CGRectMake(355, 140, 40, 10) 
                                            withFont:[UIFont systemFontOfSize:11] 
                                       lineBreakMode:UILineBreakModeTailTruncation 
                                           alignment:UITextAlignmentRight]; // Skriv måleenhet på aksen    
    
    // Tegn andreakse med måleenhet
    CGContextMoveToPoint(contxt, 10, 5); // Start ny strek fra dette punkt
    CGContextAddLineToPoint(contxt, 10, 315); // Lag en linje fra forrige punkt og hit
    CGContextStrokePath(contxt); // Og så tegner vi langs linjen
    [[NSString stringWithFormat:@"[N]"] drawInRect:CGRectMake(15, 4, 30, 10) 
                                          withFont:[UIFont systemFontOfSize:11] 
                                     lineBreakMode:UILineBreakModeTailTruncation 
                                         alignment:UITextAlignmentLeft]; // Skriv måleenhet på aksen

	// Her blir selve datagrafen tegnet - hvis det er noe data å bruke...
	if (pointCount != 0){
        // Sett farge for grafen
        CGContextSetStrokeColorWithColor(contxt, graphYColor());
		for (int i =1; i < pointCount; i++){
            // Sett startpkt for linjen
			CGContextMoveToPoint(contxt, (i*xMultiplier)+9, 160-(yValues[i-1]*yMultiplier));
            // Sett sluttpkt for linjen
			CGContextAddLineToPoint(contxt, (i*xMultiplier)+xMultiplier+9, 160-(yValues[i]*yMultiplier));
            // Og så tegner vi linjen
			CGContextStrokePath(contxt);
		}
	}
     
}

- (void)dealloc
{
    [super dealloc];
}


@end
